/*! \file common.c

  \brief common utility.

  \author
    Copyright (C) SEIKO EPSON CORPORATION 2010.

    Driver for Epson SIDM Printers

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdarg.h>
#include <ctype.h>
#include <time.h>
#include <cups/cups.h>

#include "common.h"


/*! \brief (for DEBUG only)

  Output delay count in micro sec.
  \note Set 50000 (50ms delay), When job cancel test.
 */
#define  OUTPUT_DELAY 0


/*
 * File scope global parameters...
 */

/*! \brief (for DEBUG only)

  Mamory allocate counter */
static int alloc_counter = 0;

/*! \brief (for DEBUG only)

  Mamory free counter */
static int free_counter = 0;




/* ****************************************************************
   standard C function wrapper
   **************************************************************** */

/*! Micro seconds sleep

  like a \c usleep(3)

  \param  usec  wait count (micro second)
  \return       error status
  \retval 0     completed
  \retval other error
 */
FUNCTION int safe_usleep( const unsigned long usec )
{
  int retval;
  struct timeval tv;

  if ( 0 == usec )
    return 0;

  tv.tv_sec = usec / 1000000;
  tv.tv_usec = usec % 1000000;

  retval = select(0, NULL, NULL, NULL, &tv);

  return(retval);
}



/*! \brief malloc(3) wrapper function.

    \see man \c malloc(3)
 */
FUNCTION void *my_malloc( size_t size )
{
  void *ptr = NULL;

  alloc_counter++;

  if ( 0 == size ) {
    ASSERT( 0 != size );
    return NULL;
  }

  ptr = malloc(size);

  return ptr;
}



/*! \brief free(3) wrapper function.

    \see man \c free(3)
 */
FUNCTION void my_free( void *ptr )
{
  free_counter++;

  if ( NULL == ptr ) {
    ASSERT( NULL != ptr );
    return;
  }

  free(ptr);
}


/* ****************************************************************
   unique functions
   **************************************************************** */


/*! \brief read alloc/free count

  \param num_alloc  int pointer for memory allocat counter
  \param num_free   int pointer for memory  free   counter
 */
FUNCTION void get_my_alloc_counter( int *num_alloc, int *num_free )
{
  ASSERT( NULL != num_alloc ); if ( NULL == num_alloc ) { return; }
  ASSERT( NULL != num_free  ); if ( NULL == num_free  ) { return; }

  *num_alloc = alloc_counter;
  *num_free  = free_counter;
}



/*! \brief Local round function.

  \param value  original value in \a from base.
  \param from   base unit for \a value.
  \param to     base unit for return value.
  \return       converted value from \a to base unit.
 */
FUNCTION unsigned long my_round_ul( unsigned long value, int from, int to )
{
  unsigned long retval = 0;

  ASSERT( 0 != from  ); if ( 0 == from  ) { return 0; }
  ASSERT( 0 !=  to   ); if ( 0 ==  to   ) { return 0; }

  if ( 0 == value ) {
    return 0;
  }

  retval = value * to / from;
  if ( ( (value * 10 * to / from) % 10 )>= 5 ) {
    retval = retval + 1;
  }
  return retval;
}


/*! \brief Local round function for float to unsigned long.

  \param value  original value in \a from base unit.
  \param from   base unit for \a value.
  \param to     base unit for return value.
  \return       converted value.
 */
FUNCTION unsigned long my_round_ftoul( float value, int from, int to )
{
  unsigned long retval = 0;

  ASSERT( 0 != from  ); if ( 0 == from  ) { return 0; }
  ASSERT( 0 !=  to   ); if ( 0 ==  to   ) { return 0; }

  if ( 0 >= value ) {
    return 0;
  }

  retval = value * to / from;
  if ( ( value - retval ) >= 0.5 ) {
    retval = retval + 1;
  }

  return retval;
}


/*! \brief Local round-down function for float to unsigned long.

  \param value  original value in \a from base unit.
  \param from   base unit for \a value.
  \param to     base unit for return value.
  \return       converted value.
 */
FUNCTION unsigned long my_rounddown_ftoul( float value, int from, int to )
{
  unsigned long retval = 0;

  ASSERT( 0 != from  ); if ( 0 == from  ) { return 0; }
  ASSERT( 0 !=  to   ); if ( 0 ==  to   ) { return 0; }

  if ( 0 >= value ) {
    return 0;
  }

  retval = value * to / from;

  return retval;
}



/*! \brief Check all zero value in the memory.
  \param buff     one line buffer.
  \param num      number of bytes in \a buff
  \return         empty or not.
  \retval 1       empty
  \retval 0       not empty
 */
FUNCTION int is_empty( const unsigned char *buff, unsigned int num )
{
  unsigned int i;

  ASSERT( NULL != buff ); if ( NULL == buff ) { return 1; }

  for ( i = 0; i < num; i++ ) {
    if ( 0 != buff[i] ) {
      return 0;
    }
  }
  return 1;
}

/* end of file */
