/*! \file escpcmd.c

  \brief ESC/P command output functions.

  \author
    Copyright (C) SEIKO EPSON CORPORATION 2010.

    Driver for Epson SIDM Printers

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include "option.h"
#include "output.h"
#include "escpcmd.h"
#include "debug.h"


/*! \brief ESC/P Start Job Command

  \param escp_page_info    ESC/P Page size information
  \param design_base_reso  Base resolution design unit in this driver. (for ESC ( U, ESC ( C, ESC ( c)
  \param vreso_base        Vertical resolution base unit. (for ESC ( C, ESC ( c)
  \param high_speed        User specified high-speed mode
  \param support_pse_fsx   Bitimage Print Speed Enhancer support flag (bit-field)
  \retval 0             completd
  \retval other         error
 */
FUNCTION int escp_start_job(escp_page_info_t *escp_page_info, unsigned short design_base_reso, unsigned short vreso_base, int high_speed, unsigned char support_pse_fsx)
{
  const unsigned char escp_packet_mode_auto_cmd[] = {
    0x00, 0x00, 0x00,
    ESC,  0x01,
    '@', 'E', 'J', 'L', ' ', '1', '2', '8', '4', '.', '4',  LF,
    '@', 'E', 'J', 'L', ' ', ' ', ' ', ' ', ' ', LF };
  unsigned char buf[MAX_STR_BUF];
  unsigned long lnum = 0;       /* temporary value */
  const char   *opt_print_direction = NULL;
  int           write_error = 0;

  if ( ARGERROR( NULL == escp_page_info   ) ) { return EINVAL; }
  if ( ARGERROR(    0 == design_base_reso ) ) { return EINVAL; }
  if ( ARGERROR(    0 == vreso_base       ) ) { return EINVAL; }

  if ( is_match_key_value(PPDKEYPacketMode, "Auto") ) {

    write_error |= printd_out(escp_packet_mode_auto_cmd, sizeof(escp_packet_mode_auto_cmd));
  }


  /* reset */
  write_error |= printf_out("%c%s", ESC, "@");

  /* current position move to lefthand. */
  buf[0] = CR;
  write_error |= printd_out(buf, 1);

  /* select 10cpi = 1/10 inch */
  write_error |= printf_out("%c%s", ESC, "P");


  if ( support_pse_fsx ) {
    if ( -1 != high_speed ) {
      write_error |= printf_out("%c%s", FS, "x");
      buf[0] = high_speed;
      write_error |= printd_out(buf, 1);
    }
  }


  opt_print_direction = find_key_value(PPDKEYPrintDirection);
  if ( NULL != opt_print_direction ) {
    if ( 0 != strcmp(opt_print_direction,"Default") ) {

      /* turn unidirectional mode on/off */
      write_error |= printf_out("%c%s", ESC, "U");

      if ( NULL != strstr(opt_print_direction, "UniOn") ) {
        buf[0] = 1;             /* unidirectional mode ON */
      } else {
        buf[0] = 0;             /* unidirectional mode OFF */
      }
      write_error |= printd_out(buf, 1);
    }
  }


  /* set unit
     1/360 inch : 24pin
     1/432 inch :  9pin
  */
  write_error |= printf_out("%c%s", ESC, "(U");
  buf[0] = 1;
  buf[1] = 0;
  buf[2] = SET_UNIT_VALUE_M;         /* 10 */
  write_error |= printd_out(buf, 3);


  /* set page length in defined unit */
  write_error |= printf_out("%c%s", ESC, "(C");
  buf[0] = 2;
  buf[1] = 0;

  lnum = my_round_ftoul(escp_page_info->page_length, design_base_reso, vreso_base);
  buf[2] = (unsigned char)(lnum % 256);
  buf[3] = (unsigned char)(lnum / 256);
  write_error |= printd_out(buf, 4);

  if ( is_match_key_value(PPDKEYTopMP, "Driver" ) ) {

    /* set page format in defined unit */
    write_error |= printf_out("%c%s", ESC, "(c");
    buf[0] = 4;
    buf[1] = 0;

    lnum = my_round_ftoul(escp_page_info->margin_top, design_base_reso, vreso_base);
    buf[2] = (unsigned char)(lnum % 256);
    buf[3] = (unsigned char)(lnum / 256);

    lnum = my_round_ftoul(escp_page_info->margin_bottom, design_base_reso, vreso_base);
    buf[4] = (unsigned char)(lnum % 256);
    buf[5] = (unsigned char)(lnum / 256);
    write_error |= printd_out(buf, 6);
  }

  if ( write_error ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: Print data output error.\n");
    return 1;
  }
  return 0;
}


/*! \brief ESC/P End Job Command

  \retval 0             completd
  \retval other         error
 */
FUNCTION int escp_end_job( void )
{
  int           write_error = 0;

  FUNCSTART;

  /* Auto Tear Off */
  if ( is_match_key_value(PPDKEYTearOff, "On")) {

    if ( is_match_key_value_n(PPDKEYInputSlot, "Tractor", strlen("Tractor"))) {
      write_error |= printf_out("%c%c%c%c", CR, ESC, EM, 'C');

    } else if ( is_match_key_value_n(PPDKEYInputSlot, "RollPaper", strlen("RollPaper"))) {

      if ( is_match_key_value(PPDKEYSvRP, "On")) {
        write_error |= printf_out("%c%c%c%c", CR, ESC, EM, 'D');

      } else {
        write_error |= printf_out("%c%c%c%c", CR, ESC, EM, 'C');
      }
    } else {
      write_error |= printf_out("%c%c%c", ESC, '@', CR);
    }

  } else {

    write_error |= printf_out("%c%c%c", ESC, '@', CR);
  }


  if ( write_error ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: Print data output error.\n");
    return 1;
  }
  return 0;
}


/*! \brief ESC/P Start Page Command

  \retval 0             completd
  \retval other         error
 */
FUNCTION int escp_start_page( void )
{
  const unsigned char carriage_return[1] = { CR };
  int           write_error = 0;

  /* current position move to lefthand. */
  write_error |= printd_out(carriage_return, 1);

  if ( write_error ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: Print data output error.\n");
    return 1;
  }
  return 0;
}


/*! \brief ESC/P End Page Command

  \retval 0             completd
  \retval other         error
 */
FUNCTION int escp_end_page( void )
{
  const unsigned char form_feed[1] = { FF };
  const unsigned char escp_save_roll_paper_cmd[] = {
    ESC, 'J', 0x18,
    ESC, '(', 'C', 0x02, 0x00, 0xF0, 0x1E };
  int           write_error = 0;

  if ( is_match_key_value(PPDKEYInputSlot, "RollPaper") &&
       is_match_key_value(PPDKEYSvRP, "On" ) ) {

    write_error |= printd_out(escp_save_roll_paper_cmd, sizeof(escp_save_roll_paper_cmd));

  } else {

    /* form feed */
    write_error |= printd_out(form_feed, 1);
  }

  if ( write_error ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: Print data output error.\n");
    return 1;
  }
  return 0;
}


/* end of file */
