/*! \file job.c

  \brief print job handring functions

  \author
    Copyright (C) SEIKO EPSON CORPORATION 2010.

    Driver for Epson SIDM Printers

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "job.h"
#include "escpcmd.h"
#include "debug.h"


/*
 * Define...
 */

/* CUPS printer model number. */
#define EPSON_9PIN      0       /*!< EPSON  9 pins serial impact dot matrics printer. */
#define EPSON_24PIN     1       /*!< EPSON 24 pins serial impact dot matrics printer. */


/*! \brief Start Job
  \param  job_info      job information
  \param  arg5          command-line argument No.5 (for CUPS option)
  \param  fd            a input file descriptor.
  \return               error status
  \retval 0             completed
  \retval other         error
 */
FUNCTION int start_job( job_info_t *job_info, const char *arg5, int fd )
{
  int                 num_cmd_options = 0; /* number of command-line CUPS options */
  cups_option_t      *cmd_options = NULL;  /* command-line CUPS options */
  ppd_file_t         *ppd = NULL;
  const ppd_size_t   *pageSize = NULL;

  ppd_attr_t         *support_pse_bmp = NULL;
  ppd_attr_t         *support_pse_fsx = NULL;

  int                 status = 0;
  int                 i = 0;

  FUNCSTART;

  if ( ARGERROR( NULL == job_info ) ) { return EINVAL; }
  if ( ARGERROR( NULL == arg5     ) ) { return EINVAL; }

  /* Get PPD information from CUPS */

  ppd = ppdOpenFile(getenv("PPD"));

  if ( NULL == ppd ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: Can not read PPD information.\n");
    return ENOENT;
  }

  ppdMarkDefaults(ppd);

  cmd_options = NULL;
  num_cmd_options = cupsParseOptions( arg5, 0, &cmd_options );

  /* debug: Show command-line options for CUPS. */
  if (1) { debug_show_cups_cmd_option_list(num_cmd_options, cmd_options); }


  i = cupsMarkOptions(ppd, num_cmd_options, cmd_options);
  if ( ARGERROR( 1 == i ) ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: Options conflicting.\n");
    status = EINVAL;
    goto er_start_job;
  }

  status = set_options( ppd );
  if ( 0 != status ) {
    DEBUG_ERROR_TRACE;
    goto er_start_job;
  }

  /* debug: Show current option key and value list. */
  if (1) { debug_show_option_list(); }


  if ( NULL != cmd_options ) {
    cupsFreeOptions(num_cmd_options, cmd_options);
    num_cmd_options = 0;
    cmd_options = NULL;
  }

  /* Set ESC/P base units */
  if (        EPSON_24PIN == ppd->model_number ) {
    job_info->design_base_reso = 3600;
    job_info->vmove_base = 180; /* for ESC J */
    job_info->hmove_base = 360; /* for ESC $ */

  } else if ( EPSON_9PIN  == ppd->model_number ) {
    job_info->design_base_reso = 4320;
    job_info->vmove_base = 216; /* for ESC J */
    job_info->hmove_base = 60;  /* for ESC $ */

  } else {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: The Printer is not supported.\n");
    status = EINVAL;
    goto er_start_job;
  }

  job_info->vreso_base = job_info->design_base_reso / SET_UNIT_VALUE_M;


  /* init page size parameter */

  pageSize = ppdPageSize( ppd, NULL);
  if ( NULL == pageSize ) {

    const char *page_size_name;

    DEBUGINFO;
    debug_log(dbg, "ppdPageSize() error:\ttry again ");

    page_size_name = find_key_value(PPDKEYPageSize);

    DEBUGINFO;
    debug_log(dbg, "pageSize=%s.\n", page_size_name);

    if ( NULL != page_size_name ) {
      pageSize = ppdPageSize( ppd, page_size_name );
    }

    if ( NULL == pageSize ) {
      DEBUG_ERROR_TRACE;
      fprintf(stderr, "ERROR: PageSize error.\n");
      status = -1;
      goto er_start_job;
    }
  }
  job_info->page_num = 0;

  /* copy struct ppd_size_s */
  job_info->page_size.marked     = pageSize->marked;
  memcpy(job_info->page_size.name, pageSize->name, PPD_MAX_NAME);
  job_info->page_size.width      = pageSize->width;
  job_info->page_size.length     = pageSize->length;
  job_info->page_size.left       = pageSize->left;
  job_info->page_size.bottom     = pageSize->bottom;
  job_info->page_size.right      = pageSize->right;
  job_info->page_size.top        = pageSize->top;


  support_pse_bmp = ppdFindAttr(ppd, PPDATTRSpPseBmp, NULL);
  if ( support_pse_bmp ) {
    job_info->support_pse_bmp = atoi(support_pse_bmp->value);
    debug_log(dbg, "Support Print Speed Enhancer (BMP) : %u\n", job_info->support_pse_bmp);
  }

  support_pse_fsx = ppdFindAttr(ppd, PPDATTRSpPseFsx, NULL);
  if ( support_pse_fsx ) {
    job_info->support_pse_fsx = atoi(support_pse_fsx->value);
    debug_log(dbg, "Support Print Speed Enhancer (FSx) : %u\n", job_info->support_pse_fsx);
  }


  job_info->ras = cupsRasterOpen(fd, CUPS_RASTER_READ);
  if ( ARGERROR(NULL == job_info->ras) ) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "ERROR: CUPS Fatal Error (cupsRasterOpen).\n");
    status = EIO;
    goto er_start_job;
  }

  if ( NULL != ppd ) {
    ppdClose(ppd);
    ppd = NULL;
  }

  return 0;


 er_start_job:

  DEBUG_ERROR_TRACE;

  if ( NULL == job_info->ras) {
    cupsRasterClose(job_info->ras);
    job_info->ras = NULL;
  }

  if ( NULL != cmd_options ) {
    cupsFreeOptions(num_cmd_options, cmd_options);
    num_cmd_options = 0;
    cmd_options = NULL;
  }

  if ( NULL != ppd ) {
    ppdClose(ppd);
    ppd = NULL;
  }

  return status;
}

/*
 * End of print...
 */
/*! \brief End Job
  \param  job_info      job information
  \return       error status
  \retval 0     completed
  \retval other error
 */
FUNCTION int end_job( job_info_t *job_info )
{
  int write_error = 0;

  FUNCSTART;

  if ( ARGERROR(NULL == job_info) ) { return EINVAL; }

  write_error = escp_end_job();

  if ( NULL != job_info->input_raster_buff ) {
    my_free(job_info->input_raster_buff);
    job_info->input_raster_buff = NULL;
    job_info->band_buf = NULL;
  }

  if ( NULL != job_info->ras ) {
    cupsRasterClose(job_info->ras);
    job_info->ras = NULL;
  }

  if ( write_error ) {
    DEBUG_ERROR_TRACE;
    return EIO;
  }

  return 0;
}

/* end of file */
