/*! \file rastertoepsidm.c

  \brief ESC/P, ESC/P2 filter for CUPS raster data.

  \author
    Copyright (C) SEIKO EPSON CORPORATION 2010.

    Driver for Epson SIDM Printers

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "job.h"
#include "page.h"
#include "bitimage.h"
#include "rastertoepsidm.h"
#include "debug.h"

LOCALFUNC int feed_and_send_band(job_info_t *job_info, const cups_page_header2_t *page_header, unsigned short skip_line_num, unsigned short band_line);


/*
 * File scope global parameters...
 */

/*! \brief Job Information (real memory) */
static job_info_t  _job_info = {0,};


/*
 * global parameters...
 */

/*! \brief Job Information pointer (access pointer) */
job_info_t  *job_info = &_job_info;



/*
 * MAIN PROCESS...
 */


/*! \brief rastertoepsidm

  main process.

  \param    fd            a input file descriptor.
  \param    arg5          command-line argument No.5 (for CUPS option)
  \return                 error status
  \retval   zero          completed
  \retval   other         error
*/
FUNCTION  int rastertoepsidm( int fd, const char *arg5 )
{
  cups_page_header2_t       _page_header; /* Page header (real memory) */
  const cups_page_header2_t *page_header = &_page_header; /* Page header (pointer) */
  unsigned short             skip_line_num = 0; /* number of skip lines */
  unsigned short             band_line = 0;  /* number of line in band_buf */
  int                        scan_line = 0;  /* Current line */
  int                        status = 0; /* temporary error status */
  int                        ret_status = 0; /* return status */
  unsigned char             *src = NULL;
  unsigned char             *dst = NULL;
  unsigned short             y;

  FUNCSTART;

  if ( ARGERROR( NULL == arg5 ) ) { return EINVAL; }

  /* clear job_info */
  memset(job_info, 0, sizeof(job_info_t));

  ret_status = start_job(job_info, arg5, fd);

  if ( 0 != ret_status ) {
    DEBUG_ERROR_TRACE;
    return ret_status;
  }

  /* page loop start */
  while (cupsRasterReadHeader2(job_info->ras, &_page_header)) {

    /* count up page counter */
    job_info->page_num++;
    fprintf(stderr, "PAGE: %d %d\n", job_info->page_num, page_header->NumCopies);

    if ( CUPS_CSPACE_K != page_header->cupsColorSpace ) {
      DEBUG_ERROR_TRACE;
      fprintf(stderr, "ERROR: CUPS data format error.\n");
      ret_status = -1;
      goto er_rastertoepsidm1;
    }

    ret_status = start_page(job_info, page_header);

    if ( 0 != ret_status ) {
      DEBUG_ERROR_TRACE;
      goto er_rastertoepsidm1;
    }


    band_line = 0;
    skip_line_num = 0;

    /* loop start */
    for (scan_line = 0; scan_line < page_header->cupsHeight; scan_line++) {

      if ((scan_line & 127) == 0) {
        fprintf(stderr, "INFO: Printing page %d, %d%% complete...\n", 
                job_info->page_num, 100 * scan_line / page_header->cupsHeight);
      }

      if ( 1 > cupsRasterReadPixels(job_info->ras, job_info->input_raster_buff, page_header->cupsBytesPerLine) ) {
        break;
      }

      /* debug: input raster image data output to file. */
      debug_image_out(job_info->input_raster_buff, page_header->cupsBytesPerLine);

      /* Failsafe for input of image data of illegal size from CUPS.  */
      if ( scan_line >= job_info->lines_per_page ) {
        /* NOP */
        if (1) { dbgprintf("  scan_line=%d ignore\n", scan_line); }
        continue;
      }

      if ( ( 0 == band_line ) &&
           is_empty(job_info->input_raster_buff, job_info->band_bytes_per_line) ) {

        skip_line_num++;

        if (1) { debug_show_scanline_info(scan_line, band_line, skip_line_num); }

      } else {

        src =  job_info->input_raster_buff;
        dst = &job_info->band_buf[ band_line * job_info->band_bytes_per_line ];
        memcpy(dst, src, job_info->band_bytes_per_line);

        band_line++;

        if (1) { debug_show_scanline_info(scan_line, band_line, skip_line_num); }

        if ( band_line == page_header->cupsRowCount ) {
          ret_status = feed_and_send_band(job_info, page_header, skip_line_num, band_line);
          skip_line_num = band_line;
          band_line = 0;
          if ( 0 != ret_status ) {
            DEBUG_ERROR_TRACE;
            goto er_rastertoepsidm1;
          }
        }
      }
    } /* loop end */


    if ( ARGERROR((skip_line_num + band_line) < page_header->cupsRowCount) ) {
      goto er_rastertoepsidm1;
    } else {
      if ( 0 != band_line ) {
        if ( !is_empty(job_info->band_buf, job_info->band_bytes_per_line * band_line ) ) {

          if (1) { debug_show_scanline_info(scan_line, band_line, skip_line_num); }

          skip_line_num = (skip_line_num + band_line) - page_header->cupsRowCount;

          for ( y = 0; y < band_line; y++ ) {
            src = &job_info->band_buf[ (                band_line - y - 1) * job_info->band_bytes_per_line ];
            dst = &job_info->band_buf[ (page_header->cupsRowCount - y - 1) * job_info->band_bytes_per_line ];
            memcpy(dst, src, job_info->band_bytes_per_line);
          }
          memset(job_info->band_buf, 0, (page_header->cupsRowCount - band_line) * job_info->band_bytes_per_line);
          band_line = page_header->cupsRowCount;

          debug_log(dbg, "Match the final raster to the bottom pin.\n");
          if (1) { debug_show_scanline_info(scan_line, band_line, skip_line_num); }

          ret_status = feed_and_send_band(job_info, page_header, skip_line_num, band_line);
          skip_line_num = band_line;
          if ( 0 != ret_status ) {
            DEBUG_ERROR_TRACE;
            goto er_rastertoepsidm1;
          }
        }
      }
    }

  er_rastertoepsidm1:

    status = end_page(job_info);
    if ( !ret_status ) {
      ret_status = status;
    }

    band_line = 0;
    skip_line_num = 0;

    if ( 0 != ret_status ) {
      break;
    }
  }

  status = end_job(job_info);
  if ( !ret_status ) {
    ret_status = status;
  }

  if (job_info->page_num == 0) {
    DEBUG_ERROR_TRACE;
    fprintf(stderr, "WARNING: No pages found!\n");
  } else {
    fprintf(stderr, "INFO: Ready to print.\n");
  }

  /* debug: Check memory alloc/free miss. */
  debug_check_memory_allocate_counter();

  debug_log(dbg, "\nall done.\n");

  return ret_status;
}



/*
 * Cancel the current job...
 */

/*! \brief Cancel Job
  \param  sig signal from system (CUPS).
  \return     void
 */
FUNCTION void cancel_job(int sig)
{
  long  i;

  FUNCSTART;

  dbgprintf("\n ******** JOB CANCEL ******** \n");
  debug_log(dbg," signal number=%d\n", sig);

  /*
   * Send out lots of NUL bytes to clear out any pending raster data...
   */
  debug_log(dbg, " number of send NUL chars=%u\n", job_info->bmp_data_size);
  if ( job_info ) {
    for ( i = 0; i < job_info->bmp_data_size; i++ ) {
      putchar(0);
    }
  }


  /*
   * Send out some NUL bytes for pending ESC command cancel ...
   */
  debug_log(dbg, " number of send NUL chars=%d\n", 16);
  for ( i = 0; i < 16; i++ ) {
    putchar(0);
  }

  /*
   * End the current page and exit...
   */

  end_page(job_info);
  end_job(job_info);

  /* debug: Check memory alloc/free miss. */
  debug_check_memory_allocate_counter();

  /* debug: close debug log file and image out file. */
  debug_close();

  exit(0);
}



/*! \brief line feed and send bitimage data to printer

  \param job_info       #job_info_t *
  \param page_header \c cups_page_header2_t *
  \param skip_line_num  number of line skip
  \param band_line      number of line of data in the band
  \retval 0      success
  \retval other  error

*/

LOCALFUNC int feed_and_send_band(job_info_t *job_info, const cups_page_header2_t *page_header, unsigned short skip_line_num, unsigned short band_line)
{
  bm_info_t  real_bm_info;
  bm_info_t *bm_info;
  int        ret_status = 0;

  if ( ARGERROR( NULL == job_info    ) ) { return EINVAL; }
  if ( ARGERROR( NULL == page_header ) ) { return EINVAL; }

  bm_info = &real_bm_info;

  memset(bm_info, 0, sizeof(bm_info_t));

  bm_info->bitmap_band_buf = &(job_info->bitmap_band_buf);
  bm_info->bmp_data_size   = &(job_info->bmp_data_size);

  bm_info->band_pixel_per_line = job_info->band_pixel_per_line;
  bm_info->hmove_base          = job_info->hmove_base;

  bm_info->high_speed_mode = job_info->high_speed_mode;
  bm_info->support_pse_bmp = job_info->support_pse_bmp;

  bm_info->dots_per_column = page_header->cupsRowCount;
  bm_info->HWResolution_H  = page_header->HWResolution[0];
  bm_info->HWResolution_V  = page_header->HWResolution[1];

  if ( 0 != skip_line_num ) {
    ret_status = line_feed(page_header->HWResolution[1], skip_line_num, job_info->vmove_base);
    if ( 0 != ret_status ) {
      DEBUG_ERROR_TRACE;
      return ret_status;
    }
  }

  if ( 0 != band_line ) {
    ret_status = send_band(bm_info, band_line, job_info->band_buf);
    if ( 0 != ret_status ) {
      DEBUG_ERROR_TRACE;
      return ret_status;
    }
  }
  return 0;
}


/* end of file */
